(** A few utilities for printing pretty command lines *)
open Printf;;
open Util;;

(** Get the default key from a key doc list *)
let defkey kd = (fst (List.hd kd));;
let width = ref 80;;

let normsep="\n     ";;
let symbhead="\n    ";;
let symbsep="\n       ";;

(** Produce the key list and doc list for a kd list *)
let keys_docs top kd =
  let mkcmd (key,doc) =
    let docw = wordwrap ~sep:symbsep !width doc in
    sprintf "     %s\n       %s\n" key docw in
  let def = defkey kd in
  let kds = (sprintf "%s%s default: %s\n" symbhead top def)::(List.map mkcmd kd) in
  let keys,_ = List.split kd in
  keys,(String.concat "" kds);;

(** Split a 3 way list *)
let rec split3 l =
  match l with
	[] -> [],[],[]
  | (a,b,c)::t -> 
	  let al,bl,cl = split3 t in
	  (a::al,b::bl,c::cl)

(* (\** Produce the key list, symbol list, and doc list for a kd list *\) *)
(* let keysym_docs top ksd = *)
(*   let mkcmd (key,_,doc) = *)
(*     let docw = wordwrap ~sep:symbsep !width doc in *)
(*     sprintf "     %s\n       %s\n" key docw in *)
(*   let docs = List.map mkcmd kd in *)
(*   let keys,syms,_ = split3 ksd in *)
(*   let defstr = List.hd keys in *)
(*   let topdoc = sprintf "%s%s default: %s\n" symbhead top defstr in *)
(*   keys,syms,(String.concat "" kds);; *)

let format_docs top keys docs =
  let defkey = List.hd keys in
  let topdoc = sprintf "%s%s default: %s\n" symbhead top defkey in
  let mkcmd key doc =
    let docw = wordwrap ~sep:symbsep !width doc in
    sprintf "     %s\n       %s\n" key docw in
  let formdocs = List.map2 mkcmd keys docs in
  String.concat "" (topdoc::formdocs);;

type flag = string;;
type doc = string;;
type arg = string;;

(** An extended symbol which allows better printing for the options  *)
type spec = 
    ExtSymbol of flag*doc*(string->unit)*((string*doc) list)
  | ExtArb of flag*doc*bool*((string*doc*(unit->unit)) list)
  | Ext2string of flag*arg*(string->string->unit)*doc
  | ExtStringList of flag*arg*int*(string list->unit)*doc
  | ExtOther of flag*Arg.spec*arg*doc;;

(** Create a list of options for [Arg.parse] with slightly nicer
    documentation strings *)
let make_def_options specs =
  let f s = match s with
      ExtSymbol(flag,topdoc,setf,kdlist)->
	let keys,docs = List.split kdlist in
	let formdoc = format_docs topdoc keys docs in
	setf (List.hd keys);
	flag,Arg.Symbol(keys,setf),formdoc
    | ExtArb(flag,topdoc,init,kdflist)->
	let keys,docs,funs = split3 kdflist in
	let formdoc = format_docs topdoc keys docs in
	if init then ((List.hd funs) ());
	let kflist = List.combine keys funs in
	let f s = let g = List.assoc s kflist in g () in
	flag,Arg.Symbol(keys,f),formdoc	  
    | Ext2string(flag,arg,setf,doc) ->
	let docw = wordwrap ~sep:normsep !width doc in
	let s1 = ref "" in
	flag,
	Arg.Tuple([Arg.String(fun s->s1:=s);Arg.String(fun s2->setf !s1 s2)]),
	(arg^normsep^docw^"\n")
    | ExtStringList(flag,arg,n,setf,doc) ->
	let docw = wordwrap ~sep:normsep !width doc in
	let arglist = ref [] in
	let other s = arglist:= s::!arglist in
	let final s = setf (List.rev (s::!arglist)) in
	let rec iter i = 
	  if i=1 then [Arg.String(final)] else Arg.String(other)::(iter (i-1)) in
	flag,Arg.Tuple(iter n),(arg^normsep^docw^"\n")
    | ExtOther(flag,aspec,arg,doc) ->
	let docw = wordwrap ~sep:normsep !width doc in
	flag,aspec,(arg^normsep^docw^"\n")
  in
  List.map f specs;;

let assoc_set aref lst s =(*  s :'a ref -> ('a * string) list -> string ->unit  = *)
  aref := List.assoc s lst;;
(* let alt_parse options deffn usg = *)
(*   try *)
(*     Arg.parse_argv Sys.argv options deffn usg *)
(*   with *)
(*     Arg.Bad(s) -> printf "%s\n" s; exit 1 *)
(*   | Arg.Help(_) -> Arg.usage options usg;; *)

(* 	| ExtSymbolp(flag,topdoc,setf,key_sym_doc_list) -> *)
(* 		let keys,syms,docs = split3 key_sym_doc_list in *)
(* 		setf (List.head sym); *)
(* 		let formdoc = format_docs topdoc keys docs in *)
(* 		let newsetf key = setf (List.assoc key (List.combine keys syms)) in *)
(* 		flag,Arg.Symbol(keys,newsetf),formdoc *)
