(** Data structure and a few utilities for residue pairs *)

open Util
open Printf
open Scanf

type respair = {
  pos1:int;				(** Position of first residue *)
  pos2:int;				(** Position of second residue *)
  objs:float array;			(** Objective values *)
  idxs:int array;			(** Indices in the original objectives of each term *)
}

(** Indices *)
let mm,ms,sm,ss = (0,1,2,3)

let newpair p1 p2 = {
  pos1=p1; pos2=p2; objs=Array.make 4 nan; idxs=Array.make 4 (-1);
}

let empty_respair = newpair (-1) (-1)

(** Write a residue pair to an open channel *)
let write_respair f p =
  fprintf f "%d %d %f %f %f %f %d %d %d %d" 
    p.pos1 p.pos2 
    p.objs.(mm) p.objs.(ms) p.objs.(sm) p.objs.(ss)
    p.idxs.(mm) p.idxs.(ms) p.idxs.(sm) p.idxs.(ss)

(** Read a single residue pair from an open channel *)
let read_respair f =
  let convert pos1 pos2 mm ms sm ss mmi msi smi ssi =
    {pos1=pos1; pos2=pos2; objs=[|mm;ms;sm;ss|];  idxs=[|mmi;msi;smi;ssi|];}
  in
  bscanf f " %d %d %r %r %r %r %d %d %d %d " rnan rnan rnan rnan convert

(** Read a file full of residue pairs *)
let read_respairs fname =
  let n = Util.count_lines fname in
  let f = Scanning.from_file fname in
  Array.init n (fun _-> read_respair f)
  
(** Arrange the pairs in a matrix indexed by their residue positions
    for quick access *)
let pairmap pairs n =
  let map = Array.make_matrix n n (-1) in
  Array.iteri (fun i p -> map.(p.pos1).(p.pos2) <- i) pairs;
  map

(** Arrange the pairs in a matrix indexed by their residue positions
    for quick access *)
let pairmatrix pairs n =
  let map = Array.make_matrix n n empty_respair in
  Array.iteri (fun _ p -> map.(p.pos1).(p.pos2) <- p) pairs;
  map
